unit XMLCruncherForm;

interface

uses
  ActiveX, Clipbrd , Windows, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, Dialogs, StdCtrls, XMLDoc, xmldom, XMLIntf, XPMan, Menus;

type
  TThreadCrunch = class(TThread)
  private
    procedure DoCrunch;
  protected
    procedure Execute; override;
  end;

  TForm1 = class(TForm)
    MemoStatus: TMemo;
    ButtonClose: TButton;
    OpenDialog: TOpenDialog;
    PopupMenu: TPopupMenu;
    miSelectAll: TMenuItem;
    miCopy: TMenuItem;
    miDash1: TMenuItem;
    miTransparent: TMenuItem;
    miSave: TMenuItem;
    miDash2: TMenuItem;
    miRestart: TMenuItem;
    miOpen: TMenuItem;
    miClose: TMenuItem;
    ButtonOpen: TButton;
    procedure miSelectAllClick(Sender: TObject);
    procedure miCopyClick(Sender: TObject);
    procedure miTransparentClick(Sender: TObject);
    procedure miSaveClick(Sender: TObject);
    procedure miRestartClick(Sender: TObject);
    procedure miOpenClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure miCloseClick(Sender: TObject);
    procedure ThreadOnTerminate(Sender: TObject);
  private
    Thread: TThreadCrunch;
    procedure InitCrunch;
  end;

var
  Form1: TForm1;
  Cfg: TXMLDocument;
  CfgFilename, Filename: string;

implementation

{$R *.dfm}

procedure TForm1.FormShow(Sender: TObject);
begin
  CfgFilename := StringReplace(ParamStr(0), ExtractFileExt(ParamStr(0)), '.xml',
    []);
  if not FileExists(CfgFilename) then
  begin
    Form1.MemoStatus.Text := 'Steuerdatei ' + ExtractFileName(CfgFilename) +
      ' nicht gefunden!';
    Exit;
  end;
  try
    Cfg := TXMLDocument.Create(nil);
    Cfg.ParseOptions := [poPreserveWhiteSpace];
    Cfg.LoadFromFile(CfgFilename);
    with Cfg.DOMDocument as IDOMNodeSelect do
    begin
      if selectNode('//Window/@x') <> nil then
        Left := StrToIntDef(selectNode('//Window/@x').nodeValue, Left);
      if selectNode('//Window/@y') <> nil then
        Top := StrToIntDef(selectNode('//Window/@y').nodeValue, Top);
      if selectNode('//Window/@width') <> nil then
        Width := StrToIntDef(selectNode('//Window/@width').nodeValue, Top);
      if selectNode('//Window/@height') <> nil then
        Height := StrToIntDef(selectNode('//Window/@height').nodeValue, Top);
      if selectNode('//Window/@transparent') <> nil then
        miTransparent.Checked := StrToBoolDef(selectNode('//Window/@transparent').nodeValue,
          miTransparent.Checked);
      AlphaBlend := miTransparent.Checked;
    end;
  except
    on E: Exception do
    begin
      Form1.MemoStatus.Text := 'Fehler: ' + E.Message;
      Exit;
    end;
  end;

  if (ParamCount <= 0) or (Length(ParamStr(1)) <= 0) then
  begin
    MemoStatus.Text := 'Aufruf:'#13#10 + ExtractFileName(ParamStr(0)) +
      ' Dateiname[.xml] [/l] [/q]'#13#10'/l Log-Datei schreiben'#13#10 +
      '/q Automatisch beenden';
    Exit;
  end;
  Filename := ParamStr(1);
  if StrPos(PAnsichar(Filename), '.') = nil then
    Filename := Filename + '.xml';
  // Extrahiert den Pfadnamen mit Backslash am Ende.
  Filename := ExtractFilePath(ParamStr(0)) + Filename;
  inherited;
  InitCrunch;
end;

procedure TForm1.miSelectAllClick(Sender: TObject);
begin
  MemoStatus.SelectAll;
end;

procedure TForm1.miCopyClick(Sender: TObject);
begin
  if MemoStatus.SelLength > 0 then
    Clipboard.AsText := MemoStatus.SelText
  else
    Clipboard.AsText := MemoStatus.Text;
end;

procedure TForm1.miTransparentClick(Sender: TObject);
begin
  Form1.AlphaBlend := miTransparent.Checked;
end;

procedure TForm1.miSaveClick(Sender: TObject);
begin
  with Cfg.DOMDocument as IDOMNodeSelect do
  begin
    if selectNode('//Window') = nil then
      Cfg.DOMDocument.documentElement.appendChild(Cfg.DOMDocument.createElement('Window'));
    with selectNode('//Window').attributes do
    begin
      setNamedItem(Cfg.DOMDocument.createAttribute('x'));
      setNamedItem(Cfg.DOMDocument.createAttribute('y'));
      setNamedItem(Cfg.DOMDocument.createAttribute('width'));
      setNamedItem(Cfg.DOMDocument.createAttribute('height'));
      setNamedItem(Cfg.DOMDocument.createAttribute('transparent'));
      getNamedItem('x').nodeValue := IntToStr(Left);
      getNamedItem('y').nodeValue := IntToStr(Top);
      getNamedItem('width').nodeValue := IntToStr(Width);
      getNamedItem('height').nodeValue := IntToStr(Height);
      getNamedItem('transparent').nodeValue := BoolToStr(miTransparent.Checked);
    end;
    Cfg.SaveToFile(CfgFilename);
  end;
end;

procedure TForm1.miRestartClick(Sender: TObject);
begin
  if (Thread <> nil) and not Thread.Terminated then
    Exit;
  if not FileExists(Filename) then
  begin
    miOpenClick(Sender);
    Exit;
  end;
  InitCrunch;
end;

procedure TForm1.miOpenClick(Sender: TObject);
begin
  if (Thread <> nil) and not Thread.Terminated then
    Exit;
  if not OpenDialog.Execute(Form1.Handle) then
    Exit;
  Filename := OpenDialog.FileName;
  InitCrunch;
end;

procedure TForm1.miCloseClick(Sender: TObject);
begin
  if (Thread <> nil) and not Thread.Terminated then
    Exit;
  Close;
end;

procedure TForm1.InitCrunch;
begin
  if not FileExists(Filename) then
  begin
    Form1.MemoStatus.Text := 'XML-Datei ' + ExtractFileName(Filename) +
      ' nicht gefunden!';
    Exit;
  end;

  Form1.Caption := ExtractFileName(Filename);
  Form1.MemoStatus.Text := 'Laden...';
  Form1.ButtonClose.SetFocus;
  Form1.ButtonOpen.Enabled := False;
  Form1.ButtonClose.Enabled := False;

  Thread := TThreadCrunch.Create(True);
  Thread.OnTerminate := ThreadOnTerminate;
  Thread.Resume;
end;

procedure TForm1.ThreadOnTerminate(Sender: TObject);
var
  LogFilename: string;
  FileStream: TFileStream;
begin
  Form1.ButtonOpen.Enabled := True;
  Form1.ButtonClose.Enabled := True;

  if FindCmdLineSwitch('l') then
  begin
    LogFilename := StringReplace(ParamStr(0), ExtractFileExt(ParamStr(0)),
      '.log', []);
    try
      if FileExists(LogFilename) then
        FileStream := TFileStream.Create(LogFilename, fmOpenReadWrite)
      else
        FileStream := TFileStream.Create(LogFilename, fmCreate);
      try
        FileStream.Position := FileStream.Size;
        FileStream.Write(PChar(Form1.Caption)^, Length(Form1.Caption));
        FileStream.Write(#13#10, 2);
        FileStream.Write(PChar(Form1.MemoStatus.Text)^, Length(Form1.MemoStatus.Text));
        FileStream.Write(#13#10#13#10, 4);
      finally
        FileStream.Free;
      end;
    except
    on E: Exception do
      MemoStatus.Text := E.Message;
    end;
  end;
  if FindCmdLineSwitch('q') then
    Close;
end;

{ TThreadCrunch }

procedure TThreadCrunch.Execute;
begin
  CoInitialize(nil);
  try
    DoCrunch;
    Terminate;
  finally
    CoUninitialize;
  end;
end;

procedure TThreadCrunch.DoCrunch;
var
  Map: TXMLDocument;
  CfgNodes, NodeList: IDOMNodeList;
  I, J, RemoveCount: Integer;
  FileIn, FileOut: TextFile;
  Line, CountResults, XPath: string;
begin
  try
    Map := TXMLDocument.Create(nil);
    Map.LoadFromFile(Filename);

    CountResults := '';

  RemoveCount := 0;
  Form1.MemoStatus.Text := 'Lsche...';
  with Cfg.DOMDocument as IDOMNodeSelect do
  begin
    CfgNodes := selectNodes('//Remove/@xpath');
    for I := 0 to CfgNodes.Length - 1 do
    begin
      if I mod 10 = 9 then
        Form1.MemoStatus.Text := 'Lsche ' + IntToStr(I + 1) + ' von ' +
          IntToStr(CfgNodes.Length) + '...';
      XPath := CfgNodes[I].nodeValue;
      with Map.DOMDocument as IDOMNodeSelect do
      begin
        NodeList := selectNodes(XPath);
        for J := NodeList.Length - 1 downto 0 do
        begin
          NodeList[J].parentNode.removeChild(NodeList[J]);
          Inc(RemoveCount);
        end;
      end;
    end;
  end;
  CountResults := CountResults + 'Gelschte XML-Knoten: ' +
    IntToStr(RemoveCount) + #13#10;

  RemoveCount := 0;
  with Cfg.DOMDocument as IDOMNodeSelect do
  begin
    CfgNodes := selectNodes('//Insert');
    for I := 0 to CfgNodes.Length - 1 do
    begin
      XPath := CfgNodes[I].attributes.getNamedItem('xpath').nodeValue;
      with Map.DOMDocument as IDOMNodeSelect do
      begin
        NodeList := selectNodes(XPath);
        if CfgNodes[I].attributes.getNamedItem('attribute') <> nil then
        begin
          for J := NodeList.Length - 1 downto 0 do
          begin
            if NodeList[J].attributes.getNamedItem(
              CfgNodes[I].attributes.getNamedItem('attribute').nodeValue) <> nil then
              Continue;

            NodeList[J].attributes.setNamedItem(Cfg.DOMDocument.createAttribute(
              CfgNodes[I].attributes.getNamedItem('attribute').nodeValue));

            NodeList[J].attributes.getNamedItem(
              CfgNodes[I].attributes.getNamedItem('attribute').nodeValue).nodeValue :=
                CfgNodes[I].attributes.getNamedItem('value').nodeValue;

            Inc(RemoveCount);
          end;
        end
        else if CfgNodes[I].attributes.getNamedItem('node') <> nil then
        begin
          for J := NodeList.Length - 1 downto 0 do
          begin
            NodeList[J].appendChild(Cfg.DOMDocument.createElement(
              CfgNodes[I].attributes.getNamedItem('node').nodeValue));

            if CfgNodes[I].attributes.getNamedItem('value') <> nil then
              NodeList[J].lastChild.appendChild(Cfg.DOMDocument.createTextNode(
                CfgNodes[I].attributes.getNamedItem('value').nodeValue));

            Inc(RemoveCount);
          end;
        end;
      end;
    end;
  end;
  if RemoveCount > 0 then
    CountResults := CountResults + 'Durchgefhrte Einfgungen: ' +
      IntToStr(RemoveCount) + #13#10;

  RemoveCount := 0;
  with Cfg.DOMDocument as IDOMNodeSelect do
  begin
    CfgNodes := selectNodes('//Replace');
    for I := 0 to CfgNodes.Length - 1 do
    begin
      if CfgNodes[I].attributes.getNamedItem('value') = nil then
        Continue;
      if I mod 10 = 9 then
        Form1.MemoStatus.Text := 'Ersetze ' + IntToStr(I + 1) + ' von ' +
          IntToStr(CfgNodes.Length) + '...';
      XPath := CfgNodes[I].attributes.getNamedItem('xpath').nodeValue;
      with Map.DOMDocument as IDOMNodeSelect do
      begin
        NodeList := selectNodes(XPath);
        for J := NodeList.Length - 1 downto 0 do
        begin
          NodeList[J].nodeValue :=
            CfgNodes[I].attributes.getNamedItem('value').nodeValue;
          Inc(RemoveCount);
        end;
      end;
    end;
  end;
  if RemoveCount > 0 then
    CountResults := CountResults + 'Durchgefhrte Ersetzungen: ' +
      IntToStr(RemoveCount) + #13#10;

  with Cfg.DOMDocument as IDOMNodeSelect do
  begin
    CfgNodes := selectNodes('//Count');
    for I := 0 to CfgNodes.Length - 1 do
    begin
      if I mod 10 = 9 then
        Form1.MemoStatus.Text := 'Zhle ' + IntToStr(I + 1) + ' von ' +
          IntToStr(CfgNodes.Length) + '...';
      XPath := CfgNodes[I].attributes.getNamedItem('xpath').nodeValue;
      with Map.DOMDocument as IDOMNodeSelect do
      begin
        NodeList := selectNodes(XPath);
        if NodeList.length <>
          StrToIntDef(CfgNodes[I].attributes.getNamedItem('default').nodeValue, 0)
          then
          CountResults := CountResults +
            CfgNodes[I].attributes.getNamedItem('description').nodeValue + ': '
            + IntToStr(NodeList.length) + #13#10;
      end;
    end;
  end;

    except
    on E: Exception do
    begin
      Form1.MemoStatus.Text := 'Fehler: ' + E.Message;
      Exit;
    end;
  end;

  Form1.MemoStatus.Text := 'Speichern...';
  Map.SaveToFile(Filename + '_temp.xml');
  Map.Free;

  AssignFile(FileIn, Filename + '_temp.xml');
  Reset(FileIn);
  AssignFile(FileOut, Filename + '_crunched.xml');
  Rewrite(FileOut);
  while (not Eof(FileIn)) do
  begin
    Readln(FileIn, Line);
    {while StrPos(PAnsichar(Line), #9'<') <> nil do
      Line := StringReplace(Line, #9'<', '<', [rfReplaceAll]);
    while StrPos(PAnsichar(Line), ' <') <> nil do
      Line := StringReplace(Line, ' <', '<', [rfReplaceAll]);}
    while StrPos(PAnsichar(Line), '" />') <> nil do
      Line := StringReplace(Line, '" />', '"/>', [rfReplaceAll]);
    Writeln(FileOut, Trim(Line));
  end;
  CloseFile(FileOut);
  CloseFile(FileIn);

  DeleteFile(Filename + '_temp.xml');

  Form1.MemoStatus.Text := Trim(CountResults);
end;

end.
